function [beta,err]=nma_hmmBackward(observation, symbols, initProb, emitProb, tranProb)
%
%calculate the HMM backward algorithm
%
%INPUT
%  observation: a char array of the observation sequence
%
%  symbols:     a char array. represents the alphabets allowed
%               in the observation. It is assumed the observation 
%               sequence allready contains only valid alphabets
%
%  initProb:    An array of doubles. represents the initial probability
%               of the states.
%
%  emitProb:    A matrix nxm. There are n rows for n states, and m columns for
%               m alphabets. The order of the alphabets is taken from the
%               symbols array. 
%
% tranProb:    The state transition probability matrix.
%
% OUTPUT
%   beta:      beta matrix. nxm
%   err:       empty if no errors.            
%               

err='';
beta=[];
nSeq=length(observation);

nStates=size(emitProb,1);
beta=zeros(nStates,nSeq);

for(state=1:nStates)    
   beta(state,end) = log(1);
end

for(time=nSeq-1:-1:1)
   x=observation(time+1);
   currentEmitProb=[];
   for(k=1:length(symbols))
      if(x==char(symbols(k)))
         currentEmitProb=emitProb(:,k);
         break;
      end
   end
 
   for(currentState=1:nStates)             
       beta(currentState,time)=nma_logOfSumsBeta(beta(:,time+1),tranProb(currentState,:),currentEmitProb);        
   end
end


