function [alpha,err]=nma_hmmForward(observation,symbols,initProb,emitProb,tranProb)
%function prob=nma_hmmForward(observation,h.symbols,h.initProb,h.emitProb,h.tranProb)
%
%calculate the HMM forward algorithm
%
%INPUT
%  observation: a char array of the observation sequence
%
%  symbols:     a char array. represents the alphabets allowed
%               in the observation. It is assumed the observation 
%               sequence allready contains only valid alphabets
%
%  initProb:    An array of doubles. represents the LOG_e initial probability
%               of the states. 
%
%  emitProb:    A matrix nxm. There are n rows for n states, and m columns for
%               m alphabets. The order of the alphabets is taken from the
%               symbols array. The emission probability, in Log_e 
%
% tranProb:    The state transition probability matrix. In Log_e
%
% OUTPUT
%   alpha:     Alpha matrix. nxm
%   err:       empty if no errors.            
%               

err='';

nStates=size(emitProb,1);
alpha=zeros(nStates,length(observation));

for(state=1:nStates)   
   [p,err]=nma_hmmEmitProb(state,observation(1),emitProb,symbols);
   if(~isempty(err))
      return;
   end
   
   alpha(state,1) = initProb(state) + p;
end

for(time=2:length(observation))
   x=observation(time);
   
   for(currentState=1:nStates)
      [emitProbAtCurrentState,err] = nma_hmmEmitProb(currentState,x,emitProb,symbols);
      if(~isempty(err))
         return;
      end
      
      sum=nma_logOfSums(alpha(:,time-1),tranProb(currentState,:));
      
      alpha(currentState,time)=sum + emitProbAtCurrentState;          
      
   end
end








