function nma_hmm_callbacks(arg,h0)
%function nma_hmm_callbacks(arg,h0)
%

%
% Final project
% course MATH 127, UC Berkeley, FALL 2002 tought by Dr Lior Pachter.
%

%change history
%nabbasi-112802 started
%nabbasi-0201402 all algorithms implemented OK. Need more GUI work.

switch arg 
   case 'init'
      % called by nma_alignment_main just after loading the GUI
      
      warning on;
      
      h = struct( ...
         'isHMMLoaded',0,...
         'fullHMMFileName','',...
         'hmmLastFolder','',...
         'fullLogFileName','',...
         'hmmFileName_tag',0,...
         'logFileName_tag',0,...
         'seq_tag',0,...
         'hmmFID',0,...
         'modelSeq_tag',0,...
         'modelStates_tag',0,...
         'modelStateTranMatrix_tag',0,...
         'modelEmitProbMatrix_tag',0,...
         'modelInitialProb_tag',0,...
         'forward_tag',0,...
         'forward_log_tag',0,...
         'veterbi_tag',0,...
         'grid_tag',0,...
         'forward_grid_tag',0,...
         'backwardPos_tag',0,...
         'backwardState_tag',0,...
         'backwardProb_tag',0,...
         'backwardGrid_tag',0,...
         'symbols',{''},...
         'emitProb',[],...
         'Log_Emit_Prob',[],...
         'states',{''},...
         'tranProb',[],...
         'Log_Tran_Prob',[],...
         'initProb',[],...
         'Log_Init_Prob',[],...
         'beta',[],...
         'alpha',[],...
         'gamma',[],...
         'observation','');
      
      zoom(h0,'on');
      
      %
      %  set version number to display and program name and update build date
      %
      set(h0,'Name','Simple HMM model, V 1.0. Written by Nasser Abbasi for MATH 127 course tought by Dr Lior Pachter, UC Berkeley. Fall 2002.');
      set(h0,'NumberTitle','off');
      
      %
      % store all GUI handles in userData for quick access in the callbacks
      %
      h.hmmFileName_tag       = findobj(h0,'Tag','hmmFileName_tag');
      h.logFileName_tag       = findobj(h0,'Tag','logFileName_tag');
      h.seq_tag               = findobj(h0,'Tag','seq_tag');
      h.modelSeq_tag          = findobj(h0,'Tag','modelSeq_tag');
      h.modelStates_tag       = findobj(h0,'Tag','modelStates_tag');
      h.modelStateTranMatrix_tag = findobj(h0,'Tag','modelStateTranMatrix_tag');
      h.modelEmitProbMatrix_tag  = findobj(h0,'Tag','modelEmitProbMatrix_tag');
      h.modelInitialProb_tag     = findobj(h0,'Tag','modelInitialProb_tag');
      h.forward_tag            = findobj(h0,'Tag','forward_tag');
      h.forward_log_tag        = findobj(h0,'Tag','forward_log_tag');      
      h.veterbi_tag            = findobj(h0,'Tag','veterbi_tag');
      h.grid_tag               = findobj(h0,'Tag','grid_tag');
      h.forward_grid_tag       = findobj(h0,'Tag','forward_grid_tag');
      
      h.backwardPos_tag      = findobj(h0,'Tag','backwardPos_tag');
      h.backwardState_tag    = findobj(h0,'Tag','backwardState_tag');
      h.backwardProb_tag     = findobj(h0,'Tag','backwardProb_tag');
      h.backwardGrid_tag     = findobj(h0,'Tag','backwardGrid_tag');
      
      
      set(h0,'UserData',h);
      
   case 'backward_callback'
      
      [o,FF]=gcbo;       
      h=get(FF,'UserData');
      
      if(~h.isHMMLoaded)
         uiwait(errordlg(sprintf('Error. No HMM model loaded. Please load an HMM model first')));
         return;                      
      end
      
      observation= get(h.seq_tag,'String');
      observation=cleanSeq(observation); 
      if(length(observation)==0)
         uiwait(errordlg(sprintf('Error. No observation sequence available. Please type in a sequence.')));
         return;                      
      end
      
      theTime = get(h.backwardPos_tag,'String');
      theTime   = str2double(theTime);
      if( isnan(theTime))
         uiwait(errordlg(sprintf('Error in backward algorithm input. Invalid numeric value for symbol position. [%s]',theTime)));
         return;                      
      end         
      
      if(theTime<1 | theTime>length(observation))
         if(theTime<1)
            uiwait(errordlg(sprintf('Error in backward algorithm input. can not have a negative value for symbol position.')));                         
         else
            uiwait(errordlg(sprintf('Error in backward algorithm input. position given exceeds length of sequence')));                        
         end         
         return;
      end
      
      inputState = nma_trim(get(h.backwardState_tag,'String'));
      if(isempty(inputState))
         uiwait(errordlg(sprintf('Error in backward algorithm input. Empty state name.')));                     
         return;
      end
      
      found=0;
      for(i=1:length(h.states))
         s=char(h.states(i));
         if(length(s)==length(inputState))
            if(s==inputState)
               found=1;
               break;
            end
         end          
      end
      
      if(~found)
         uiwait(errordlg(sprintf('Error in backward algorithm input. Invalid state name. Unrecgnized. check spelling')));                              
         return;
      end
      
      statePos=getPosOfState(h.states,inputState);    
      prob=exp(h.beta(statePos,theTime))*exp(h.alpha(statePos,theTime))/fullProb(h);
      set(h.backwardProb_tag,'String',prob);
      
   case 'run_callback'
      
      [o,FF]=gcbo;       
      h=get(FF,'UserData');
      
      if(~h.isHMMLoaded)
         uiwait(errordlg(sprintf('Error. No HMM model loaded. Please load an HMM model first')));
         return;                      
      end
      
      observation= get(h.seq_tag,'String');
      observation=cleanSeq(observation); 
      if(length(observation)==0)
         uiwait(errordlg(sprintf('Error. No observation sequence available. Please type in a sequence.')));
         return;                      
      end
      
      set(h.seq_tag,'String',observation);
      
      h.observation=observation;
      
      [h,err]=forward(h);
      if(~isempty(err))
         uiwait(errordlg(sprintf('%s',err)));    
         return;
      end
      
      h=viterbi(h);
      
      [beta,err]=nma_hmmBackward(observation,h.symbols,h.Log_Init_Prob,h.Log_Emit_Prob,h.Log_Tran_Prob);
      if(~isempty(err))
         uiwait(errordlg(sprintf('%s',err)));    
         return;
      end      
      h.beta=beta;
      
      Formatted_String = Format_Probability_Grid( h.beta, h.states , []);                             
      Position_String  = Format_Position_String( h.alpha );     
      Final_String     = Append_Cell_Array( Formatted_String, Position_String);
      
      set( h.backwardGrid_tag, 'String', Final_String);
      
      set(FF,'UserData',h);
      return;                     
      
      
   case 'openHMMFile_callback'
      
      [o,FF]=gcbo;       
      h=get(FF,'UserData');
      
      if(~isempty(h.hmmLastFolder))
         cd( h.hmmLastFolder);
      end
      
      [fileName, pathName] = uigetfile(['*.hmm;*.txt'], 'Select HMM configuration file to open');        
      if(pathName==0)
         return;
      end
      
      fullFileName=[pathName fileName];
      
      %         [fid,err] = fopen(fullFileName,'rt');
      %         if(~isempty(err))
      %             uiwait(errordlg(sprintf('Error opening HMM file[%s]. error=%s.',fullFileName,err)));
      %             h.hmmFID = 0;
      %             set(FF,'UserData',h);
      %             return;
      %         end
      %         
      %         fclose(fid);
      %         
      h.hmmLastFolder=pathName;
      h.fullHMMFileName=fullFileName;
      
      set(h.hmmFileName_tag,'String',fullFileName);       
      
      [symbols, initProb, states, tranProb, emitProb, err]=nma_readHMM(h.fullHMMFileName);
      if(~isempty(err))
         uiwait(errordlg(sprintf('Error parsing HMM configuration file[%s]. error=%s.',fileName,err)));
         return;
      end
      
      DELTA=0.00001;
      LOW_THRESHOLD=1-DELTA;
      HIGH_THRESHOLD=1+DELTA;
      
      T=sum(initProb);
      if(T<LOW_THRESHOLD | T>HIGH_THRESHOLD)
         uiwait(errordlg(sprintf('Initial probability sum is found to be %f. It must equal 1.0. Correct the configuration file',T)));
         return;                      
      end
      
      Number_Of_States = size(tranProb,1);
      for(row=1:Number_Of_States)
         T=sum(tranProb(row,:));
         if(T<LOW_THRESHOLD | T>HIGH_THRESHOLD)          
            uiwait(errordlg(sprintf('Probability transition matrix, row %d,  the sum must equal 1.0, found %f Correct the configuration file',row,T)));
            return;                      
         end                                
      end
      
      for(row=1:Number_Of_States)
         T=sum(emitProb(row,:));
         if(T<LOW_THRESHOLD | T>HIGH_THRESHOLD)          
            uiwait(errordlg(sprintf('emit probability matrix, row %d,  the sum must equal 1.0, found %f Correct the configuration file',row,T)));
            return;                      
         end                                
      end
      
      h.isHMMLoaded=1;
      
      h.symbols=symbols;
      h.initProb=initProb;
      h.emitProb=emitProb;
      h.states=states;
      h.tranProb=tranProb;
      
      % convert prob to log now for speed, so we do not have
      % to do this every time.
      Number_Of_States  = length(states);
      Number_Of_Symbols = length(symbols);
      
      for(i=1:Number_Of_States)
         if(initProb(i) == 0 )
            h.Log_Init_Prob(i) = -inf;
         else 
            h.Log_Init_Prob(i) = log(initProb(i));
         end                        
         
         for(j=1:Number_Of_Symbols)
            if(emitProb(i,j)==0)
               h.Log_Emit_Prob(i,j) = -inf;   
            else                  
               h.Log_Emit_Prob(i,j) = log(emitProb(i,j));   
            end
         end
         
         for(j=1:Number_Of_States)
            if(tranProb(i,j)==0)
               h.Log_Tran_Prob(i,j) = -inf;
            else                  
               h.Log_Tran_Prob(i,j) = log(tranProb(i,j));   
            end               
         end
      end
      
      T='';
      symbolsAsString=char(symbols);
      for(i=1:length(symbols))
         if(i==length(symbols))             
            T=[T sprintf('%c',symbolsAsString(i))];
         else
            T=[T sprintf('%c,',symbolsAsString(i))];
         end                
      end
      
      set(h.modelSeq_tag,'String',{T});
      
      set(h.modelStates_tag,'String',states);
      
      theText={''};
      nLine=0;
      T='';
      statesAsString=char(states);
      for(j=1:size(tranProb,2))
         T=[T sprintf('%10c  %s',' ',statesAsString(j,:))];
      end
      nLine=nLine+1;
      theText(nLine)={T};
      
      for(i=1:size(tranProb,1))
         T=sprintf('%s  ',statesAsString(i,:));
         for(j=1:size(tranProb,2))
            T=[T sprintf('%6.6f  ',tranProb(i,j))];
         end
         nLine=nLine+1;
         theText(nLine)={T};
      end
      
      set(h.modelStateTranMatrix_tag,'String',theText);
      
      theText={''};
      nLine=0;
      T='';
      for(i=1:size(emitProb,1))
         T='';
         for(j=1:size(emitProb,2))
            T=[T '    ' sprintf('%6.6f',emitProb(i,j))];
         end
         nLine=nLine+1;
         theText(nLine)={T};
      end
      
      set(h.modelEmitProbMatrix_tag,'String',theText);
      set(h.modelInitialProb_tag,'String',initProb);
      
      set(FF,'UserData',h);
      return;
      
      
   case 'openLogFile_callback'
      
      [o,FF]=gcbo;       
      h=get(FF,'UserData');
      
      if(~isempty(h.hmmLastFolder))
         cd( h.hmmLastFolder);
      end
      
      [fileName, pathName] = uigetfile(['*.txt'], 'Select log file to open');        
      if(pathName==0)
         return;
      end
      
      fullFileName=[pathName fileName];
      
      [fid,err] = fopen(fullFileName,'at');
      if(~isempty(err))
         uiwait(errordlg(sprintf('Error opening log file[%s]. error=%s.',fullFileName,err)));
         h.hmmFID = 0;
         set(FF,'UserData',h);
         return;
      end
      
      fclose(fid);
      
      h.hmmLastFolder=pathName;
      h.fullLogFileName=fullFileName;
      
      set(h.logFileName_tag,'String',fullFileName);           
      
      set(FF,'UserData',h);
      return;                    
end

%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%
function sum=fullProb(h)
sum=0;
for(i=1:size(h.alpha,1))
   sum=sum+exp(h.alpha(i,end));
end

%%%%%%%%%%%%%%%%%%%%%%%%%
%
% removes spaces
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Output_Sequence=cleanSeq(seq)

Number_Of_Lines = size(seq,1);
Length_Of_Line  = size(seq,2);

Output_Sequence = zeros(Number_Of_Lines * Length_Of_Line , 1);

k=0;

for(i=1:Number_Of_Lines)   
   for(j=1:Length_Of_Line)
      if(seq(i,j)==' ' |seq(i,j)=='\t')
         ;
      else
         k=k+1;
         Output_Sequence(k)=seq(i,j);
      end
   end  
end

Output_Sequence = char(Output_Sequence(1:k))';
Output_Sequence = nma_trim(Output_Sequence);

%%%%%%%%%%%%%%%%%%%%%%%%%%%
%check if char in observation sequence is valid
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%
function valid=validSymbol(c,allowed)

valid=0;

for(i=1:length(allowed))
   if(c == char(allowed(i)))   
      valid=1;
      return;
   end
end


%%%%%%%%%%%%%%%%%%%%%%%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%
function [h,err]=forward(h)

err='';      

for(i=1:length(h.observation))
   if(~validSymbol(h.observation(i),h.symbols))
      err=sprintf('Error. observation sequence contains an invalid character ''%c''',h.observation(i));
      return;
   end
end

[alpha,err]=nma_hmmForward(h.observation,h.symbols,h.Log_Init_Prob,h.Log_Emit_Prob,h.Log_Tran_Prob);
if(~isempty(err))
   err=sprintf('Error. Failed in forward algorithm. err=%s',err);
   return;
end

h.alpha=alpha;
prob=0;
for(state=1:size(alpha,1))
   prob=prob+exp(alpha(state,end));
end

set(h.forward_tag,'String',sprintf('%g',prob));

Formatted_String = Format_Probability_Grid( h.alpha, h.states, [] );                        
Sum_Of_Probability_String = Format_Sum_Of_Probability_String(h.alpha);            
Position_String  = Format_Position_String(h.alpha);

Final_String = Append_Cell_Array( Formatted_String , Sum_Of_Probability_String);
Final_String = Append_Cell_Array( Final_String , Position_String);

set(h.forward_grid_tag,'String',Final_String);

%%%%%%%%%%%%%%%%%%%%%
%
%
%%%%%%%%%%%%%%%%%%%%%%
function h=viterbi(h)

[statePath,stateSeq,gamma]=nma_hmm_veterbi(h.observation,h.symbols,h.Log_Init_Prob,h.Log_Emit_Prob,h.Log_Tran_Prob);
h.gamma=gamma;

final={''};
k=0;
for(i=1:length(stateSeq))
   if(i==1)
      T=char(h.states(stateSeq(i)));
   else
      T=[T ',' char(h.states(stateSeq(i)))];
   end
   if(0==mod(i,20))
      k=k+1;
      final(k)={T};
      T='';
   end
end
k=k+1;
final(k)={T};

set(h.veterbi_tag,'String',final);

Formatted_String = Format_Probability_Grid(h.gamma, h.states, stateSeq);                        
Sum_Of_Probability_String = Format_Sum_Of_Probability_String(h.alpha);            
Position_String  = Format_Position_String(h.alpha);

Final_String = Append_Cell_Array( Formatted_String , Sum_Of_Probability_String);
Final_String = Append_Cell_Array( Final_String , Position_String);                   

set(h.grid_tag,'String',Final_String);

%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Formatted_String = Format_Probability_Grid(Data_Matrix, HMM_States,Viterbi_State_Seq)

n=0;
Formatted_String={''};
T='Probabilites in logs';
n=n+1;
Formatted_String(n)={T};

for(state=1:size(Data_Matrix,1))         
   name=char(HMM_States(state));
   L=length(name);
   T='';
   
   for(m=1:10)
      if(m>L)
         c=' ';
      else
         c=name(m);
      end
      T=[T c];
   end
   
   T=[T ':'];
   
   for(time=1:size(Data_Matrix,2))
      c=' ';
      if( (~isempty(Viterbi_State_Seq)) &  (Viterbi_State_Seq(time)==state) )
         c='*';
      end
      
      if(Data_Matrix(state,time)==-Inf)
         Z=sprintf('    -Inf     ');
      else
         Z=sprintf('%10.10f',Data_Matrix(state,time));
      end
      
      if(time==size(Data_Matrix,2))
         T=sprintf('%s%s%c',T,Z,c);
      else
         T=sprintf('%s%s%c    ',T,Z,c);              
      end            
   end
   
   n=n+1;
   Formatted_String(n)={T};
end

T=' ';
n=n+1;
Formatted_String(n)={T};

T='Probabilites';
n=n+1;
Formatted_String(n)={T};

for(state=1:size(Data_Matrix,1))         
   name=char(HMM_States(state));
   L=length(name);
   T='';
   for(m=1:10)
      if(m>L)
         c=' ';
      else
         c=name(m);
      end
      T=[T c];
   end
   T=[T ':'];
   for(time=1:size(Data_Matrix,2))
      if(Data_Matrix(state,time)==-Inf)
         Z=sprintf('    0        ');
      else
         Z=sprintf('+%3.10f',exp(Data_Matrix(state,time)));
      end
      
      if(time==size(Data_Matrix,2))
         T=sprintf('%s%s%c',T,Z,' ');
      else
         T=sprintf('%s%s%c    ',T,Z,' ');              
      end            
   end
   n=n+1;
   Formatted_String(n)={T};
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Sum_Of_Probability_String =  Format_Sum_Of_Probability_String(Data_Matrix)

Line_Number     = 0;

Line=' ';
Line_Number  = Line_Number + 1;
Sum_Of_Probability_String(Line_Number) = {Line};


Line='Column Sums of Probabilites';
Line_Number  = Line_Number + 1; 
Sum_Of_Probability_String(Line_Number) = {Line};


%Now also display the probability of partial sequences.
Line='prob.     :';
for(time=1:size(Data_Matrix,2))
   prob=0;
   for(state=1:size(Data_Matrix,1))
      prob=prob+exp(Data_Matrix(state,time));
   end
   
   Line=sprintf('%s+%10.10f     ',Line,prob);
end

Line_Number  = Line_Number + 1;
Sum_Of_Probability_String(Line_Number)={Line};



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Position_String = Format_Position_String(Data_Matrix)

Line_Number = 0;

Line=' ';
Line_Number  = Line_Number + 1;
Position_String(Line_Number) = {Line};

Line='Position  :';

for(time=1:size(Data_Matrix,2))
   Z=sprintf(' %12d',time);                
   Line=sprintf('%s%s%c    ',Line,Z,' ');              
end

Line_Number  = Line_Number + 1;
Position_String(Line_Number) = {Line};



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function   S=Append_Cell_Array(S1,S2)

S=S1;
N=length(S);
for(i=1:length(S2))
   S(N+i)=S2(i);   
end

