function [statePath,stateSeq,gamma]=nma_hmm_veterbi(observation,symbols,initProb,emitProb,tranProb)
%function gamma=nma_hmm_veterbi(observation,initProb,emitProb,tranProb)
%
%
%Find Viterbi Gamma values.
%
% INPUT
%    observation: the observation sequence. a Vector of characters.
%    symbols:     the allowed alphabets of the language, or observation
%    initProb:    a vector of the initial probabilities for each state
%                 in logs.
%    emitProb:    a matrix that represents the emission probability of each
%                 symbol from each state
%    tanProb:     a matrix that represents the state transition
%                 probability.
%
% OUTPUT
%   gamma:        the gamma for each state and each position. a matrix.
%   stateSeq:     the state sequence vector of most likely states.

%by Nasser Abbasi


nState=size(tranProb,1);
statePath=zeros(nState,length(observation));
gamma=zeros(nState,length(observation));

c=observation(1);
theMax=-Inf;

%initial
for(state=1:nState)
   [emit,err]=nma_hmmEmitProb(state,c,emitProb,symbols);
   if(~isempty(err))
      return;
   end
   
   gamma(state,1)= initProb(state) + emit;   
   
   if(gamma(state,1)>theMax)
      theMax=gamma(state,1);
   end   
end

for(time=2:length(observation))
   c=observation(time);
   
   for(currentState=1:nState)
      
      [emit,err]=nma_hmmEmitProb(currentState,c,emitProb,symbols);
      if(~isempty(err))
         return;
      end
      
      theMax=-Inf;      
      
      for(k=1:nState)
         t= tranProb(k,currentState) + gamma(k,time-1);
         
         if(t>theMax)
            theMax=t;
         end      
      end   
      
      %gamma(currentState,time)=theMax*emit;
      gamma(currentState,time)=theMax+ emit;
   end
end


for(time=2:length(observation))
   for(to=1:nState)
      theMax=-Inf;
      for(from=1:nState)
         t= tranProb(from,to) +gamma(from,time-1);
         
         if(t>theMax)
            theMax=t;
            statePath(to,time)=from;
         end      
      end
   end
end

stateSeq=zeros(length(observation),1);
[V,I]=max(gamma(:,end));
stateSeq(end)=I;   
for(i=length(observation)-1:-1:1)
   stateSeq(i)=statePath(stateSeq(i+1),i+1);
end
